package classwork;

import javax.media.opengl.*;
import jocode.*;

/**
 * GLART_5_video_texture.java
 * <P>
 * Create a texture from video frames.  Uses the glTexSubImage2D() function to copy 
 * new pixels into the texture each frame.  To speed up the image capture don't invert
 * the Y axis of the incoming video frame.  The video image will be upside down, but 
 * we can flip the image on the geometry (rotate the sphere 180 degrees around X) so
 * it will appear correctly in our scene.
 * <P>
 * See JOApp.copyPixelsToTexture() for the glTexSubImage2D() command.
 * <P>
 * See JMyron_Cap.java for instructions on setting up JMyron video capture libe.
 * <P>
 * napier --at-- potatoland --dot-- org
 */
public class GLART_5_video_texture extends JOApp {
    // Handle for texture that holds video image
    int videoFrameTexture = 0;    
    // a video capture object
    JMyron_Cap video = new JMyron_Cap();
    
 
    public static void main(String args[]) {
    	GLART_5_video_texture demo = new GLART_5_video_texture();
    	displayWidth = 800;
    	displayHeight = 600;
        demo.run();  // will call init(), render(), mouse functions
    }

    /**
     * Initialize the scene. 
     */
    public void setup()
    {
        // settings
        // Turn off lighting
        gl.glEnable(GL.GL_LIGHTING);
        gl.glDisable(GL.GL_BLEND);
        gl.glDisable(GL.GL_CULL_FACE);
        gl.glEnable(GL.GL_TEXTURE_2D);

        // Create a blank texture to hold the video frame
        // In draw() we'll copy pixels from the video frame into the texture
        videoFrameTexture = makeTexture(320,240);
        
        // create a light
        setLight( GL.GL_LIGHT1,
        		new float[] {1f, 1f, .8f, 1f},      // diffuse color (direct light)
        		new float[] {.8f, .8f, .7f, 1f},    // ambient color (light scattered in environment)
        		new float[] {1f, 1f, 1f, 1f},       // specular color (reflected highlight, same as direct)
        		new float[] {-6f, 6f, 4, 1f}        // position 
        		);

        // create and activate material
        setMaterial(
        		new float[] {.8f, .8f, .8f, 1f},    // whhte material
        		.6f);              // medium shiny (0=matte 1=glossy)

        // get the OpenGL version (just FYI)
        String glversion = gl.glGetString(GL.GL_VERSION);
        System.out.println("OpenGL version is " + glversion);
    }

    public void draw() {
     	rotation += 2f;

     	// copy the video frame into the texture
    	if( video.newPixels )
    	{
    		copyPixelsToTexture(video.pixelBuffer,video.rw,video.rh,videoFrameTexture);
     		video.newPixels = false;
    	}

        // Clear screen and depth buffer
        gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);

        // Select The Modelview Matrix (controls model orientation)
        gl.glMatrixMode(GL.GL_MODELVIEW);

        // Reset the Modelview matrix
        // this resets the coordinate system to center of screen
        gl.glLoadIdentity();

        // Where is the 'eye'
        glu.gluLookAt(
            0f, 1f, 4f,    // eye position
            0f, 0f, 0f,    // target to look at
            0f, 1f, 0f);   // which way is up

        // enable texture (or call JOApp.activateTexture(handle))
        gl.glBindTexture(GL.GL_TEXTURE_2D, videoFrameTexture);

        // draw large sphere at center, turning slowly on its axis
        gl.glPushMatrix();
		{
			gl.glRotatef(rotation*.2f, 0, 1, 0);
			// flip the sphere 180 degrees so the video image appears upright
			gl.glRotatef(180, 1,0,0);
			renderSphere(24);
		}
        gl.glPopMatrix();

		// draw orbiting sphere
        gl.glPushMatrix();
		{
			gl.glRotatef(rotation * .2f, 0, 1, 0);   // rotate coord system (so earth orbits sun)
			gl.glTranslatef(1.5f, 0, 0);             // shift coordinate system right
			gl.glRotatef(rotation * 1.3f, 0, 1, 0);  // rotate coord system (to spin earth)
			gl.glScalef(.3f, .3f, .3f);              // shrink it down
			// flip the sphere 180 degrees so the video image appears upright
			gl.glRotatef(180, 1,0,0);
			renderSphere(24);
		}
	    gl.glPopMatrix();
    }

    /**
     * Run() calls this right before exit.
     */
    public void exit() {
    	video.stop();  // stop the video capture
    	super.exit();
    }

}