package classwork;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.nio.IntBuffer;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import javax.swing.Timer;
import JMyron.*;  // video capture library is in JMyron.jar

/**
 * Capture video frames using the JMyron library. 
 * Grab the pixelBuffer to get latest video frame.
 * 
 * NOTE: to use the JMyron video capture library:
 * 1) put the JMyron.jar and JMyron.dll files into the <project>/lib folder
 * 2) refresh the Eclipse project
 * 3) right-click the project name on the left sidebar, 
 * 		choose Properties, Java Build Path, Libraries tab, Add Jars button
 * 		find the JMyron.jar file in the project/lib folder and add it
 * 4) click the + next to JMyron.jar 
 *		click the Native Library Location
 *		click Edit button
 *		put in the path <project>/lib
 * 5) Put the DSVL.dll and myron_ezcam.dll files into the \Windows\System32 folder
 * 		see this thread for more details about the system folder on OS X: 
 * 		http://processing.org/discourse/yabb2/YaBB.pl?board=Syntax;action=display;num=1203799374
 */
public class JMyron_Cap
{
	JMyron m;	  		// camera object
	int width  = 320;	// video frame size
	int height = 240;
	int rx = 0;			// region position and size
	int ry = 0;
	int rw = width;
	int rh = height;
	int frameRate = 15;
	boolean newPixels = false;
    ByteBuffer pixelBuffer;

	public JMyron_Cap()
	{
		init(320,240);
		//init(640,480);
	}
	
	public JMyron_Cap(int videoW, int videoH)
	{
		init(videoW,videoH);  // seems to only work with 320x240
	}
	
	public void init(int videoW, int videoH)
	{
		width = videoW;
		height = videoH;

		m = new JMyron();		//make a new instance of the object
		m.start(width,height);	//start a capture at 320x240
		m.findGlobs(0);			//disable the intelligence to speed up frame rate
		System.out.println("Myron " + m.version());
		
		setCaptureRegion(0,0,width,height);
		
		Timer videoTimer = new Timer(1000/frameRate, new ActionListener()
				{
					public void actionPerformed(ActionEvent evt)
					{
						captureFrame();
					}
				});
		videoTimer.start();
	}
	
	/** 
	 * stop the video capture, release the device
	 *
	 */
	public void stop() 
	{
		m.stop();
	}
	
	/**
	 * 
	 */
	public void setCaptureRegion(int regionX, int regionY, int regionW, int regionH) {
		// realloc pixel buffer sized to this region
		pixelBuffer = allocBytes(regionW * regionH * 4);
		rx = regionX;
		ry = regionY;
		rw = regionW;
		rh = regionH;
	}

	/**
	 * copy the video frame pixels to ByteBuffer
	 */
	private void captureFrame()
	{
		int[] img;    // video pixel array
		IntBuffer ibuffer = pixelBuffer.asIntBuffer();  // treat byte array as integers
		ibuffer.clear();
		newPixels = true;

		m.update();       //update the camera view
		img = m.image();  //get the camera frame

		
		// TEMP test sort frame
		if (false) {
			img = sortPixels(img, 0, img.length-1);
			//for (int x=0; x < width; x++) {
			//	sortPixelsColumn(img, 0, height-1, x, width);
			//}
		}
		
		// if pixelBuffer is same size as video frame, just bulk copy the ARGB pixels
		if (pixelBuffer.capacity() == img.length*4) {
			pixelBuffer.asIntBuffer().put(img).flip();
		}
		else {
			//System.out.println("asdfasdfasdf");
			// copy a sub-region into buffer
			int rx2 = rx + rw;   // outer bounds of region
			int ry2 = ry + rh;
			int ridx = 0;
			for (int i=0; i < img.length; i++) {
				int x = i % width;
				int y = i / width;
				/*
				// Used to move bytes into RGBA format (for OpenGL 1.1)
				// Now have ByteBuffer store ints as little-endian (BGRA)
				if (x >= rx && x < rx2 && y >= ry && y < ry2) {
					pixelBuffer.put(ridx++, (byte) (img[i] >> 16 & 0xFF) );   // R
					pixelBuffer.put(ridx++, (byte) (img[i] >> 8 & 0xFF) );    // G
					pixelBuffer.put(ridx++, (byte) (img[i] & 0xFF) );         // B
					pixelBuffer.put(ridx++, (byte) 1 );                       // A
				}
				*/
				if (x >= rx && x < rx2 && y >= ry && y < ry2) {
					ibuffer.put(img[i]);      // store ARGB as integer
				}
				
			}
		}
	}
	

	
	
	
	
	
	/**
	 * return a float indicating the brightness of the given color.
	 * Black returns 0, white returns 1.  Green is seen as brighter than red, red is brighter than blue.
	 */
	public float brightness(int color)
	{
		float[] rgb = color_int_float3(color);
		return (float)Math.sqrt(
		  rgb[0] * rgb[0] * .241f +    // red contributes some
		  rgb[1] * rgb[1] * .691f +    // green contributes more
		  rgb[2] * rgb[2] * .068f);    // blue contributes little
	}
	
	public float[] color_int_float3(int color) {
		return new float[] {
			(float)((color >> 16) & 0xFF)/255f,  //red
			(float)((color >> 8) & 0xFF)/255f,   //green
			(float)((color >> 0) & 0xFF)/255f    //blue
			};
	}

    /**
     * Z sort a pixel array.
     *   pixels = sortPixels(pixels, 0, pixels.length-1);
     */
    public int[] sortPixels(int[] pxls, int L, int R)
    {
     float m = (brightness(pxls[L]) + brightness(pxls[R]))/2f;
     int i = L;
     int j = R;

     do {
      while (brightness(pxls[i]) > m) i++;
      while (brightness(pxls[j]) < m) j--;

      if (i <= j) {
       int temp = pxls[i];
       pxls[i]  = pxls[j];
       pxls[j]  = temp;
       i++;
       j--;
      }
     }
     while (j >= i);

     if (L < j) sortPixels(pxls,L,j);
     if (R > i) sortPixels(pxls,i,R);
     return pxls;
    }

    /**
     * Z sort a column in a pixel array.
     *
     * for (int x=0; x < img.width; x++) {
     *   sortPixels(pixels, 0, img.height-1, x, img.width);
     * }
     */
    public int[] sortPixelsColumn(int[] pxls, int L, int R, int X, int W)
    {
     float m = (brightness(pxls[L*W+X]) + brightness(pxls[R*W+X]))/2f;
     int i = L;
     int j = R;

     do {
      while (brightness(pxls[i*W+X]) > m) i++;
      while (brightness(pxls[j*W+X]) < m) j--;

      if (i <= j) {
       int temp    = pxls[i*W+X];
       pxls[i*W+X] = pxls[j*W+X];
       pxls[j*W+X] = temp;
       i++;
       j--;
      }
     }
     while (j >= i);

     if (L < j) sortPixelsColumn(pxls,L,j,X,W);
     if (R > i) sortPixelsColumn(pxls,i,R,X,W);
     return pxls;
    }
    
    
    
    
    
	/**
	 * Allocate a bytebuffer in LITTLE_ENDIAN order.  Java is BIG_ENDIAN by default, I'm switching
	 * the order to be more compatible with graphics cards (I hope more efficient).
	 * @param howmany  number of bytes to allocate
	 * @return  the allocated ByteBuffer
	 */
    public ByteBuffer allocBytes(int howmany) {
        return ByteBuffer.allocateDirect(howmany).order(ByteOrder.LITTLE_ENDIAN);
    }
}
